const {assert} = require('chai');
const PublicationInformation = require('../PublicationInformation.js');

function timeout(ms) {
  return new Promise(resolve => setTimeout(resolve, ms));
}

const goodFetch = [{
  platform: 'nationals',
  publicationId: 2,
  internalName: 'mirror',
  names: ['Mirror Online'],
  urls: ['.mirror.co.uk']
}];

describe('PublicationInformation.js', () => {
  it('exists', () => {
    assert.exists(PublicationInformation);
  });
  it('is a class', () => {
    assert.isOk(PublicationInformation);
    assert.isFunction(PublicationInformation);
  });
  it('will redownload new publication info', async function() {
    let fetchResponse = goodFetch;
    const pubInfo = new PublicationInformation(function () {
      return {
        ok: true,
        text: async() => JSON.stringify(fetchResponse)
      };
    });
    await timeout(50);
    assert.lengthOf(pubInfo.clone(), 1);
    pubInfo._updatePublicationInformation([]);
    assert.lengthOf(pubInfo.clone(), 0);
    await pubInfo._triggerPublicationMappingRedownload();
    assert.lengthOf(pubInfo.clone(), 1);
  });
  it('retains old pub info on weird redownload response', async function()  {
    const fetchResponse = 'Not an array!';
    const pubInfo = new PublicationInformation(async function()  {
      return {
        ok: true,
        text: async() => JSON.stringify(fetchResponse)
      };
    });
    await timeout(50);
    assert.isAtLeast(pubInfo.clone().length, 2);
  });
  it('retains old pub info on failed redownload response', async function()  {
    const fetchResponse = goodFetch;
    const pubInfo = new PublicationInformation(async function()  {
      return {
        ok: false,
        text: async() => JSON.stringify(fetchResponse)
      };
    });
    await timeout(50);
    assert.isAtLeast(pubInfo.clone().length, 2);
  });
  describe('findHostname', () => {
    const pubInfo = new PublicationInformation(async function() {
      return {
        ok: true,
        text: async() => JSON.stringify(goodFetch)
      };
    });
    it('is a function', function() {
      assert.exists(pubInfo.findHostname);
      assert.isOk(pubInfo.findHostname);
      assert.isFunction(pubInfo.findHostname);
    });
    it('can find a normal hostname', function() {
      assert.isOk(pubInfo.findHostname('www.mirror.co.uk'));
      assert.isOk(pubInfo.findHostname('blah.mirror.co.uk'));
    });
    it('fails to find a bad hostname', function() {
      assert.isUndefined(pubInfo.findHostname('www.google.co.uk'));
      assert.isUndefined(pubInfo.findHostname('notAWebsite'));
      assert.isUndefined(pubInfo.findHostname(undefined));
    });
    it('finds common duplicate sites', function() {
      assert.isOk(pubInfo.findHostname('mirror.co.uk.trem.media'));
      assert.isOk(pubInfo.findHostname('www-mirror-co-uk.translate.goog'));
    });

  });
});
