const {assert} = require('chai');
const api = require('../index.js');

describe('index.js', () => {
  describe('default', () => {
    const pubMapper = api;
    it('exists', () => {
      assert.exists(pubMapper);
    });
    it('is a function', () => {
      assert.isOk(pubMapper);
      assert.isFunction(pubMapper);
    });
    it('doesn\'t crash on bad urls', () => {
      assert.isNotOk(pubMapper('asjkdhlkjsdfhlksjf'));
    });
    it('returns nothing for empty urls', () => {
      assert.isNotOk(pubMapper(''));
    });
    it('returns nothing for non reach urls', () => {
      assert.isNotOk(pubMapper('https://theguardian.com'));
    });
    it('pulls out the homepage', () => {
      assert.deepOwnInclude(pubMapper('https://www.mirror.co.uk'), {
        platform: 'nationals',
        publicationId: 2,
        internalName: 'mirror',
        names: ['Mirror Online', 'Mirror'],
        url: '.mirror.co.uk'
      });
    });
    it('pulls out the homepage from bad sections', () => {
      assert.deepOwnInclude(pubMapper('https://www.birminghammail.co.uk/somethingBad'), {
        names: ['Birmingham Live', 'Birmingham Mail'],
        platform: 'regionals',
        internalName: 'birminghammail',
        publicationId: 1,
        url: '.birminghammail.co.uk'
      });
    });
    it('pulls out an article', () => {
      assert.deepOwnInclude(pubMapper('https://www.hulldailymail.co.uk/news/hull-east-yorkshire-news/two-men-describe-desperate-moment-1309654?1232434=3463463456&/a/b#987098/a/b/c'), {
        platform: 'ronionals',
        publicationId: 6,
        channel: 'news',
        articleId: '1309654',
        section: 'hull-east-yorkshire-news',
        fullSection: 'news/hull-east-yorkshire-news',
        internalName: 'hulldailymail',
        names: ['Hull Live', 'Hull Daily Mail'],
        url: '.hulldailymail.co.uk'
      });
      assert.deepOwnInclude(pubMapper('https://www.corkbeo.ie/news/local-news/full-ireland-south-recount-resumes-16374325'), {
        platform: 'regionals',
        publicationId: 55,
        channel: 'news',
        articleId: '16374325',
        section: 'local-news',
        fullSection: 'news/local-news',
        internalName: 'corklive',
        names: ['Cork Beo'],
        url: '.corkbeo.ie'
      });
    });
    it('does not set \'articleId\' when url does not end in valid article id', () => {
      assert.notDeepOwnInclude(pubMapper('https://www.corkbeo.ie/news/local-news/full-ireland-south-recount-resumes00000000'), {
        articleId: '00000000'
      });
      assert.notDeepOwnInclude(pubMapper('https://www.corkbeo.ie/news/local-news/full-ireland-south-recount-resumes-0000a0000'), {
        articleId: '00000000'
      });
      assert.notDeepOwnInclude(pubMapper('https://www.corkbeo.ie/news/local-news/full-ireland-south-recount-resumes-00000000a'), {
        articleId: '00000000'
      });
    });
    it('pulls out a section', () => {
      assert.deepOwnInclude(pubMapper('https://www.birminghammail.co.uk/whats-on/food-drink-news/'), {
        platform: 'regionals',
        publicationId: 1,
        channel: 'whats-on',
        section: 'food-drink-news',
        fullSection: 'whats-on/food-drink-news',
        internalName: 'birminghammail',
        names: ['Birmingham Live', 'Birmingham Mail'],
        url: '.birminghammail.co.uk'
      });
    });
    it('pulls out a channel', () => {
      assert.deepOwnInclude(pubMapper('https://www.rsvplive.ie/news/'), {
        platform: 'nationals',
        publicationId: 11,
        channel: 'news',
        section: 'news',
        fullSection: 'news',
        internalName: 'rsvp',
        names: ['RSVP Live'],
        url: '.rsvplive.ie'
      });
    });
    it('doesn\'t accidentally cache the section results internally..', () => {
      assert.deepOwnInclude(pubMapper('https://www.rsvplive.ie/news/'), {
        platform: 'nationals',
        publicationId: 11,
        channel: 'news',
        section: 'news',
        fullSection: 'news',
        internalName: 'rsvp',
        names: ['RSVP Live'],
        url: '.rsvplive.ie'
      });
      assert.deepOwnInclude(pubMapper('https://www.rsvplive.ie/'), {
        platform: 'nationals',
        publicationId: 11,
        internalName: 'rsvp',
        names: ['RSVP Live'],
        url: '.rsvplive.ie'
      });
    });
    it('gets the correct info for amp pageviews', () => {
      assert.deepOwnInclude(pubMapper('https://www.mirror.co.uk/news/uk-news/hanna-yusuf-dead-talented-bbc-20377173.amp'), {
        platform: 'nationals',
        articleId: '20377173',
        publicationId: 2,
        channel: 'news',
        section: 'uk-news',
        fullSection: 'news/uk-news',
        internalName: 'mirror',
        names: ['Mirror Online', 'Mirror'],
        url: '.mirror.co.uk'
      });
    });
    it('works for demo curiously site', function() {
      const result = pubMapper('https://curiously.nav-web-dev.tm-dev-awx.com/news/bits-your-brain-ai-could-3180');
      assert.isOk(result);
    });
  });
  describe('nameLookUp', () => {
    const nameLookUp = api.nameLookUp;
    it('gets the correct info from a name lookup', () => {
      assert.deepOwnInclude(nameLookUp('mirror online'), {
        platform: 'nationals',
        publicationId: 2,
        internalName: 'mirror',
        names: ['Mirror Online', 'Mirror'],
        url: '.mirror.co.uk'
      });
      assert.deepOwnInclude(nameLookUp('rsvp live'), {
        platform: 'nationals',
        publicationId: 11,
        internalName: 'rsvp',
        names: ['RSVP Live'],
        url: '.rsvplive.ie'
      });
    });
    it('should give me null|undefined|other when i give it nonsense', () => {
      assert.equal(nameLookUp('AKJFLKJGERL UISJCNOSIJC'), null);

    });
    it('Does not return a reference to the same object in multiple calls', () => {
      const firstReturn = nameLookUp('mirror online');
      firstReturn.stuff = 'something';
      assert.notDeepEqual(nameLookUp('mirror online'), firstReturn);
    });
  });
  describe('publicationInfo', () => {
    it('Can access the full list of platform info', () => {
      const publicationInfo = api.publicationInfo();
      assert.equal(typeof publicationInfo[0], 'object');
    });
  });
  describe('idLookUp', () => {
    const idLookUp = api.idLookUp;
    it('Can look up publications using an id', () => {
      assert.deepOwnInclude(idLookUp('nationals', 2), {
        platform: 'nationals',
        publicationId: 2,
        names: ['Mirror Online', 'Mirror'],
        internalName: 'mirror',
        url: '.mirror.co.uk'
      });
      assert.deepOwnInclude(idLookUp('regionals', 2), {
        platform: 'regionals',
        publicationId: 2,
        names: ['Manchester Evening News', 'MEN'],
        internalName: 'men',
        url: '.manchestereveningnews.co.uk'
      });
      assert.deepOwnInclude(idLookUp('ronionals', 2), {
        platform: 'ronionals',
        publicationId: 2,
        names: ['Bristol Live', 'Bristol Post'],
        internalName: 'bristolpost',
        url: '.bristolpost.co.uk'
      });
    });
    it('Doesn\'t crash when it fails to find a publication by id', () => {
      assert.deepEqual(idLookUp('nationals', 99999), undefined);
      assert.deepEqual(idLookUp('notAPlatform', 2), undefined);
    });
  });
});
